# IMPORTANT
# After changing this file, run `python3 -m gama_config.generate_schemas`
# To re-generate the json schemas

from dataclasses import dataclass
from enum import Enum
from dacite import from_dict, Config
from typing import Optional, Any, List, Annotated
from gama_config.helpers import write_config, read_config, find_gama_config, join_lines
from dc_schema import SchemaAnnotation


GAMA_VESSEL_FILE_NAME = "gama_vessel.yml"
GAMA_VESSEL_SCHEMA_URL = (
    "https://greenroom-robotics.github.io/gama/schemas/gama_vessel.schema.json"
)


class Mode(str, Enum):
    SIMULATOR = "simulator"
    HARDWARE = "hardware"
    STUBS = "stubs"


class Network(str, Enum):
    SHARED = "shared"
    VPN = "vpn"
    HOST = "host"


class Variant(str, Enum):
    WHISKEY_BRAVO = "whiskey_bravo"
    EDUCAT = "educat"


class LogLevel(str, Enum):
    INFO = "info"
    DEBUG = "debug"


@dataclass
class GamaVesselConfigExtensions:
    lookout: bool = False
    rviz: bool = False
    groot: bool = False


@dataclass
class Pipeline:
    # This will become the name of frame-id, ros topic and webrtc steam
    name: str
    # Used to order the stream in the UI
    order: Optional[int]
    # An array of gstream source / transform elements
    # eg)
    # ["v4l2src", "video/x-raw, format=RGB,width=1920,height=1080"]
    elements: List[str]


@dataclass
class GamaVesselGreenstream:
    pipeline_overrides: Annotated[
        Optional[List[Optional[Pipeline]]],
        SchemaAnnotation(
            description=join_lines(
                "A list of greenstream pipelines.",
                "These will only take affect if the the mode is 'hardware'",
                "",
                "Set these to 'null' to ignore the override.",
            ),
            examples=[
                join_lines(
                    "- null",
                    "- name: fwd",
                    "  elements:" "  - v4l2src",
                    "  - video/x-raw, format=RGB,width=1920,height=1080",
                )
            ],
        ),
    ] = None


@dataclass
class GamaVesselConfig:
    variant: Variant = Variant.WHISKEY_BRAVO
    mode: Mode = Mode.SIMULATOR
    extensions: GamaVesselConfigExtensions = GamaVesselConfigExtensions()
    network: Network = Network.SHARED
    prod: bool = False
    log_level: LogLevel = LogLevel.INFO
    ubiquity_user: Optional[str] = None
    ubiquity_pass: Optional[str] = None
    ubiquity_ip: Optional[str] = None
    ddsrouter_groundstation_ip: Optional[str] = None
    ddsrouter_groundstation_port: Optional[str] = None
    ddsrouter_vessel_ip: Optional[str] = None
    ddsrouter_vessel_port: Optional[str] = None
    greenstream: GamaVesselGreenstream = GamaVesselGreenstream()


def parse_vessel_config(config: dict[str, Any]) -> GamaVesselConfig:
    return from_dict(
        GamaVesselConfig,
        config,
        config=Config(cast=[Mode, Network, Variant, LogLevel]),
    )


def get_vessel_config_path():
    return find_gama_config() / GAMA_VESSEL_FILE_NAME


def read_vessel_config() -> GamaVesselConfig:
    return read_config(get_vessel_config_path(), parse_vessel_config)


def write_vessel_config(config: GamaVesselConfig):
    return write_config(get_vessel_config_path(), config, GAMA_VESSEL_SCHEMA_URL)



from dataclasses import asdict
from enum import Enum
import yaml
import os
from typing import Any, Callable
from pathlib import Path

type_parse = Callable[[Any], Any]


def join_lines(*lines: str) -> str:
    return "\n".join(lines)


def find_gama_config() -> Path:
    """Returns the path to the .gama directory"""
    path = Path.cwd()
    while len(path.parts) > 1:
        if (path / ".gama").exists():
            return path / ".gama"
        else:
            path = path.parent

    print("Could not find a .gama directory. Are you in a GAMA project?")
    return Path("./.gama")


def dacite_to_dict(obj: Any):
    def dict_factory(data: Any):
        def convert_value(obj: Any):
            if isinstance(obj, Enum):
                return obj.value
            return obj

        return {k: convert_value(v) for k, v in data}

    return asdict(obj, dict_factory=dict_factory)


def write_config(path: Path, config: Any, schema_url: str):
    # Make the parent dir if it doesn't exist
    os.makedirs(path.parent, exist_ok=True)
    with open(path, "w") as stream:
        print(f"Writing {path}...")
        headers = f"# yaml-language-server: $schema={schema_url}"
        data = "\n".join([headers, yaml.dump(dacite_to_dict(config))])
        stream.write(data)


def read_config(path: Path, parse: type_parse):
    with open(path) as stream:
        return parse(yaml.safe_load(stream))



# IMPORTANT
# After changing this file, run `python3 -m gama_config.generate_schemas`
# To re-generate the json schemas

from dataclasses import dataclass
from enum import Enum
from dacite import from_dict, Config
from typing import Any, Optional
from gama_config.helpers import write_config, read_config, find_gama_config


GAMA_GS_FILE_NAME = "gama_gs.yml"
GAMA_GS_SCHEMA_URL = "https://greenroom-robotics.github.io/gama/schemas/gama_gs.schema.json"


class Mode(str, Enum):
    NONE = "none"
    XBOX = "xbox"
    THRUSTMASTER = "thrustmaster"
    THRUSTMASTER_COMBO = "thrustmaster_combo"
    WARTHOG = "warthog"
    WARTHOG_COMBO = "warthog_combo"


class Network(str, Enum):
    SHARED = "shared"
    VPN = "vpn"
    HOST = "host"


class LogLevel(str, Enum):
    INFO = "info"
    DEBUG = "debug"


@dataclass
class GamaGsConfig:
    mode: Mode = Mode.NONE
    network: Network = Network.SHARED
    prod: bool = False
    log_level: LogLevel = LogLevel.INFO
    remote_cmd_override: bool = False
    ddsrouter_groundstation_ip: Optional[str] = None
    ddsrouter_groundstation_port: Optional[str] = None
    ddsrouter_vessel_ip: Optional[str] = None
    ddsrouter_vessel_port: Optional[str] = None


def parse_gs_config(config: dict[str, Any]) -> GamaGsConfig:
    return from_dict(GamaGsConfig, config, config=Config(cast=[Mode, Network, LogLevel]))


def get_gs_config_path():
    return find_gama_config() / GAMA_GS_FILE_NAME


def read_gs_config() -> GamaGsConfig:
    return read_config(get_gs_config_path(), parse_gs_config)


def write_gs_config(config: GamaGsConfig):
    return write_config(get_gs_config_path(), config, GAMA_GS_SCHEMA_URL)
